@@ -1147,7 +1147,7 @@ impl Column {
 
     pub fn into_frame(self) -> DataFrame {
         // SAFETY: A single-column dataframe cannot have length mismatches or duplicate names
-        unsafe { DataFrame::new_no_checks(self.len(), vec![self]) }
+        unsafe { DataFrame::new_unchecked(self.len(), vec![self]) }
     }
 
     pub fn extend(&mut self, other: &Column) -> PolarsResult<&mut Self> {
