@@ -63,7 +63,7 @@ impl DataFrame {
         } else if by.iter().any(|s| s.dtype().is_object()) {
             #[cfg(feature = "object")]
             {
-                let mut df = DataFrame::new(by.clone()).unwrap();
+                let mut df = DataFrame::new(self.height(), by.clone()).unwrap();
                 let n = df.height();
                 let rows = df.to_av_rows();
                 let iter = (0..n).map(|i| rows.get(i));
@@ -81,11 +81,12 @@ impl DataFrame {
                 .cloned()
                 .collect::<Vec<_>>();
             if by.is_empty() {
-                let groups = if self.is_empty() {
+                let groups = if self.height() == 0 {
                     vec![]
                 } else {
                     vec![[0, self.height() as IdxSize]]
                 };
+
                 Ok(GroupsType::new_slice(groups, false, true))
             } else {
                 let rows = if multithreaded {
@@ -115,9 +116,9 @@ impl DataFrame {
     pub fn group_by<I, S>(&self, by: I) -> PolarsResult<GroupBy<'_>>
     where
         I: IntoIterator<Item = S>,
-        S: Into<PlSmallStr>,
+        S: AsRef<str>,
     {
-        let selected_keys = self.select_columns(by)?;
+        let selected_keys = self.select_to_vec(by)?;
         self.group_by_with_series(selected_keys, true, false)
     }
 
@@ -126,9 +127,9 @@ impl DataFrame {
     pub fn group_by_stable<I, S>(&self, by: I) -> PolarsResult<GroupBy<'_>>
     where
         I: IntoIterator<Item = S>,
-        S: Into<PlSmallStr>,
+        S: AsRef<str>,
     {
-        let selected_keys = self.select_columns(by)?;
+        let selected_keys = self.select_to_vec(by)?;
         self.group_by_with_series(selected_keys, true, true)
     }
 }
@@ -158,7 +159,7 @@ impl DataFrame {
 /// // create rain series
 /// let s2 = Series::new("rain".into(), [0.2, 0.1, 0.3, 0.1, 0.01]);
 /// // create a new DataFrame
-/// let df = DataFrame::new(vec![s0, s1, s2]).unwrap();
+/// let df = DataFrame::new_infer_height(vec![s0, s1, s2]).unwrap();
 /// println!("{:?}", df);
 /// ```
 ///
@@ -238,14 +239,10 @@ impl<'a> GroupBy<'a> {
         &mut self.groups
     }
 
-    pub fn take_groups(self) -> GroupPositions {
+    pub fn into_groups(self) -> GroupPositions {
         self.groups
     }
 
-    pub fn take_groups_mut(&mut self) -> GroupPositions {
-        std::mem::take(&mut self.groups)
-    }
-
     pub fn keys_sliced(&self, slice: Option<(i64, usize)>) -> Vec<Column> {
         #[allow(unused_assignments)]
         // needed to keep the lifetimes valid for this scope
@@ -311,18 +308,19 @@ impl<'a> GroupBy<'a> {
         let keys = self.keys();
 
         let agg_col = match &self.selected_agg {
-            Some(selection) => self.df.select_columns_impl(selection.as_slice()),
+            Some(selection) => self.df.select_to_vec(selection),
             None => {
                 let by: Vec<_> = self.selected_keys.iter().map(|s| s.name()).collect();
                 let selection = self
                     .df
+                    .columns()
                     .iter()
                     .map(|s| s.name())
                     .filter(|a| !by.contains(a))
                     .cloned()
                     .collect::<Vec<_>>();
 
-                self.df.select_columns_impl(selection.as_slice())
+                self.df.select_to_vec(selection.as_slice())
             },
         }?;
 
@@ -364,7 +362,8 @@ impl<'a> GroupBy<'a> {
             agg.rename(new_name);
             cols.push(agg);
         }
-        DataFrame::new(cols)
+
+        DataFrame::new_infer_height(cols)
     }
 
     /// Aggregate grouped series and compute the sum per group.
@@ -402,7 +401,7 @@ impl<'a> GroupBy<'a> {
             agg.rename(new_name);
             cols.push(agg);
         }
-        DataFrame::new(cols)
+        DataFrame::new_infer_height(cols)
     }
 
     /// Aggregate grouped series and compute the minimal value per group.
@@ -439,7 +438,7 @@ impl<'a> GroupBy<'a> {
             agg.rename(new_name);
             cols.push(agg);
         }
-        DataFrame::new(cols)
+        DataFrame::new_infer_height(cols)
     }
 
     /// Aggregate grouped series and compute the maximum value per group.
@@ -476,7 +475,7 @@ impl<'a> GroupBy<'a> {
             agg.rename(new_name);
             cols.push(agg);
         }
-        DataFrame::new(cols)
+        DataFrame::new_infer_height(cols)
     }
 
     /// Aggregate grouped `Series` and find the first value per group.
@@ -513,7 +512,7 @@ impl<'a> GroupBy<'a> {
             agg.rename(new_name);
             cols.push(agg);
         }
-        DataFrame::new(cols)
+        DataFrame::new_infer_height(cols)
     }
 
     /// Aggregate grouped `Series` and return the last value per group.
@@ -550,7 +549,7 @@ impl<'a> GroupBy<'a> {
             agg.rename(new_name);
             cols.push(agg);
         }
-        DataFrame::new(cols)
+        DataFrame::new_infer_height(cols)
     }
 
     /// Aggregate grouped `Series` by counting the number of unique values.
@@ -587,7 +586,7 @@ impl<'a> GroupBy<'a> {
             agg.rename(new_name);
             cols.push(agg);
         }
-        DataFrame::new(cols)
+        DataFrame::new_infer_height(cols)
     }
 
     /// Aggregate grouped [`Series`] and determine the quantile per group.
@@ -617,7 +616,7 @@ impl<'a> GroupBy<'a> {
             agg.rename(new_name);
             cols.push(agg);
         }
-        DataFrame::new(cols)
+        DataFrame::new_infer_height(cols)
     }
 
     /// Aggregate grouped [`Series`] and determine the median per group.
@@ -639,7 +638,7 @@ impl<'a> GroupBy<'a> {
             agg.rename(new_name);
             cols.push(agg);
         }
-        DataFrame::new(cols)
+        DataFrame::new_infer_height(cols)
     }
 
     /// Aggregate grouped [`Series`] and determine the variance per group.
@@ -652,7 +651,7 @@ impl<'a> GroupBy<'a> {
             agg.rename(new_name);
             cols.push(agg);
         }
-        DataFrame::new(cols)
+        DataFrame::new_infer_height(cols)
     }
 
     /// Aggregate grouped [`Series`] and determine the standard deviation per group.
@@ -665,7 +664,7 @@ impl<'a> GroupBy<'a> {
             agg.rename(new_name);
             cols.push(agg);
         }
-        DataFrame::new(cols)
+        DataFrame::new_infer_height(cols)
     }
 
     /// Aggregate grouped series and compute the number of values per group.
@@ -707,7 +706,7 @@ impl<'a> GroupBy<'a> {
             ca.rename(new_name);
             cols.push(ca.into_column());
         }
-        DataFrame::new(cols)
+        DataFrame::new_infer_height(cols)
     }
 
     /// Get the group_by group indexes.
@@ -741,7 +740,7 @@ impl<'a> GroupBy<'a> {
         let new_name = fmt_group_by_column("", GroupByMethod::Groups);
         column.rename(new_name);
         cols.push(column.into_column());
-        DataFrame::new(cols)
+        DataFrame::new_infer_height(cols)
     }
 
     /// Aggregate the groups of the group_by operation into lists.
@@ -779,7 +778,7 @@ impl<'a> GroupBy<'a> {
             agg.rename(new_name);
             cols.push(agg);
         }
-        DataFrame::new(cols)
+        DataFrame::new_infer_height(cols)
     }
 
     fn prepare_apply(&self) -> PolarsResult<DataFrame> {
@@ -790,9 +789,9 @@ impl<'a> GroupBy<'a> {
             } else {
                 let mut new_cols = Vec::with_capacity(self.selected_keys.len() + agg.len());
                 new_cols.extend_from_slice(&self.selected_keys);
-                let cols = self.df.select_columns_impl(agg.as_slice())?;
+                let cols = self.df.select_to_vec(agg.as_slice())?;
                 new_cols.extend(cols);
-                Ok(unsafe { DataFrame::new_no_checks(self.df.height(), new_cols) })
+                Ok(unsafe { DataFrame::new_unchecked(self.df.height(), new_cols) })
             }
         } else {
             Ok(self.df.clone())
@@ -818,29 +817,48 @@ impl<'a> GroupBy<'a> {
             .collect::<PolarsResult<Vec<_>>>()?;
 
         let mut df = accumulate_dataframes_vertical(dfs)?;
-        df.as_single_chunk_par();
+        df.rechunk_mut_par();
         Ok(df)
     }
 
     /// Apply a closure over the groups as a new [`DataFrame`].
-    pub fn apply<F>(&self, mut f: F) -> PolarsResult<DataFrame>
+    pub fn apply<F>(&self, f: F) -> PolarsResult<DataFrame>
+    where
+        F: FnMut(DataFrame) -> PolarsResult<DataFrame> + Send + Sync,
+    {
+        self.apply_sliced(None, f)
+    }
+
+    pub fn apply_sliced<F>(&self, slice: Option<(i64, usize)>, mut f: F) -> PolarsResult<DataFrame>
     where
         F: FnMut(DataFrame) -> PolarsResult<DataFrame> + Send + Sync,
     {
         let df = self.prepare_apply()?;
-        let dfs = self
-            .get_groups()
-            .iter()
-            .map(|g| {
-                // SAFETY:
-                // groups are in bounds
-                let sub_df = unsafe { take_df(&df, g) };
-                f(sub_df)
-            })
-            .collect::<PolarsResult<Vec<_>>>()?;
+        let max_height = if let Some((offset, len)) = slice {
+            offset.try_into().unwrap_or(usize::MAX).saturating_add(len)
+        } else {
+            usize::MAX
+        };
+        let mut height = 0;
+        let mut dfs = Vec::with_capacity(self.get_groups().len());
+        for g in self.get_groups().iter() {
+            // SAFETY: groups are in bounds.
+            let sub_df = unsafe { take_df(&df, g) };
+            let df = f(sub_df)?;
+            height += df.height();
+            dfs.push(df);
+
+            // Even if max_height is zero we need at least one df, so check
+            // after first push.
+            if height >= max_height {
+                break;
+            }
+        }
 
         let mut df = accumulate_dataframes_vertical(dfs)?;
-        df.as_single_chunk_par();
+        if let Some((offset, len)) = slice {
+            df = df.slice(offset, len);
+        }
         Ok(df)
     }
 
@@ -962,7 +980,7 @@ mod test {
         );
         let s1 = Column::new(PlSmallStr::from_static("temp"), [20, 10, 7, 9, 1]);
         let s2 = Column::new(PlSmallStr::from_static("rain"), [0.2, 0.1, 0.3, 0.1, 0.01]);
-        let df = DataFrame::new(vec![s0, s1, s2]).unwrap();
+        let df = DataFrame::new_infer_height(vec![s0, s1, s2]).unwrap();
 
         let out = df.group_by_stable(["date"])?.select(["temp"]).count()?;
         assert_eq!(
@@ -1026,8 +1044,10 @@ mod test {
         let s11 = Column::new("G11".into(), ["(", ")", "@", "@", "$"].as_ref());
         let s12 = Column::new("G12".into(), ["-", "_", ";", ";", ","].as_ref());
 
-        let df =
-            DataFrame::new(vec![s0, s1, s2, s3, s4, s5, s6, s7, s8, s9, s10, s11, s12]).unwrap();
+        let df = DataFrame::new_infer_height(vec![
+            s0, s1, s2, s3, s4, s5, s6, s7, s8, s9, s10, s11, s12,
+        ])
+        .unwrap();
 
         // Use of deprecated `sum()` for testing purposes
         #[allow(deprecated)]
@@ -1071,7 +1091,7 @@ mod test {
         columns.push(agg_series);
 
         // Create the dataframe with the computed series.
-        let df = DataFrame::new(columns).unwrap();
+        let df = DataFrame::new_infer_height(columns).unwrap();
 
         // Use of deprecated `sum()` for testing purposes
         #[allow(deprecated)]
