@@ -45,6 +45,14 @@ pub enum IRAggExpr {
         input: Node,
         propagate_nans: bool,
     },
+    MinBy {
+        input: Node,
+        by: Node,
+    },
+    MaxBy {
+        input: Node,
+        by: Node,
+    },
     Median(Node),
     NUnique(Node),
     Item {
@@ -167,7 +175,9 @@ impl From<IRAggExpr> for GroupByMethod {
             Std(_, ddof) => GroupByMethod::Std(ddof),
             Var(_, ddof) => GroupByMethod::Var(ddof),
             AggGroups(_) => GroupByMethod::Groups,
-            Quantile { .. } => unreachable!(),
+
+            // Multi-input aggregations.
+            Quantile { .. } | MinBy { .. } | MaxBy { .. } => unreachable!(),
         }
     }
 }
@@ -185,6 +195,11 @@ pub enum AExpr {
         options: ExplodeOptions,
     },
     Column(PlSmallStr),
+    /// Struct field value in a `struct.with_fields` context.
+    ///
+    /// Equivalent of `pl.field(name)`.
+    #[cfg(feature = "dtype-struct")]
+    StructField(PlSmallStr),
     Literal(LiteralValue),
     BinaryExpr {
         left: Node,
@@ -204,6 +219,7 @@ pub enum AExpr {
         expr: Node,
         idx: Node,
         returns_scalar: bool,
+        null_on_oob: bool,
     },
     SortBy {
         expr: Node,
@@ -244,6 +260,11 @@ pub enum AExpr {
 
         variant: EvalVariant,
     },
+    #[cfg(feature = "dtype-struct")]
+    StructEval {
+        expr: Node,
+        evaluation: Vec<ExprIR>,
+    },
     Function {
         /// Function arguments
         /// Some functions rely on aliases,
@@ -318,6 +339,8 @@ impl AExpr {
             AExpr::Eval { expr, variant, .. } => {
                 variant.is_length_preserving() && is_scalar_ae(*expr, arena)
             },
+            #[cfg(feature = "dtype-struct")]
+            AExpr::StructEval { expr, .. } => is_scalar_ae(*expr, arena),
             AExpr::Sort { expr, .. } => is_scalar_ae(*expr, arena),
             AExpr::Gather { returns_scalar, .. } => *returns_scalar,
             AExpr::SortBy { expr, .. } => is_scalar_ae(*expr, arena),
@@ -331,6 +354,8 @@ impl AExpr {
             | AExpr::Column(_)
             | AExpr::Filter { .. }
             | AExpr::Slice { .. } => false,
+            #[cfg(feature = "dtype-struct")]
+            AExpr::StructField(_) => false,
         }
     }
 
@@ -361,6 +386,8 @@ impl AExpr {
         match self {
             AExpr::Element => true,
             AExpr::Column(_) => true,
+            #[cfg(feature = "dtype-struct")]
+            AExpr::StructField(_) => true,
 
             // Over and Rolling implicitly zip with the context and thus should always be length
             // preserving
@@ -394,11 +421,14 @@ impl AExpr {
             AExpr::Eval { expr, variant, .. } => {
                 variant.is_length_preserving() && is_length_preserving_ae(*expr, arena)
             },
+            #[cfg(feature = "dtype-struct")]
+            AExpr::StructEval { expr, .. } => is_length_preserving_ae(*expr, arena),
             AExpr::Sort { expr, .. } => is_length_preserving_ae(*expr, arena),
             AExpr::Gather {
                 expr: _,
                 idx,
                 returns_scalar,
+                null_on_oob: _,
             } => !returns_scalar && is_length_preserving_ae(*idx, arena),
             AExpr::SortBy { expr, by, .. } => broadcasting_input_length_preserving(
                 std::iter::once(*expr).chain(by.iter().copied()),
