@@ -14,6 +14,8 @@ impl AExpr {
 
         match self {
             Element | Column(_) | Literal(_) | Len => {},
+            #[cfg(feature = "dtype-struct")]
+            StructField(_) => {},
             BinaryExpr { left, op: _, right } => {
                 container.extend([*right, *left]);
             },
@@ -78,6 +80,12 @@ impl AExpr {
                 _ = evaluation;
                 container.extend([*expr]);
             },
+            #[cfg(feature = "dtype-struct")]
+            StructEval { expr, evaluation } => {
+                // Evaluation is included. In case this is not allowed, use `inputs_rev_strict()`.
+                container.extend(evaluation.iter().rev().map(ExprIR::node));
+                container.extend([*expr]);
+            },
             Slice {
                 input,
                 offset,
@@ -88,6 +96,32 @@ impl AExpr {
         }
     }
 
+    /// Push the inputs of this node to the given container, in reverse order.
+    /// This ensures the primary node responsible for the name is pushed last.
+    ///
+    /// Unlike `inputs_rev`, this excludes Eval expressions. These use an extended schema,
+    /// determined by their input, which implies a different traversal order.
+    ///
+    /// This is subtly different from `children_rev` as this only includes the input expressions,
+    /// not expressions used during evaluation.
+    pub fn inputs_rev_strict<E>(&self, container: &mut E)
+    where
+        E: Extend<Node>,
+    {
+        use AExpr::*;
+
+        match self {
+            #[cfg(feature = "dtype-struct")]
+            StructEval { expr, evaluation } => {
+                // Evaluation is explicitly excluded. It is up to the caller to handle
+                // any tree traversal if required.
+                _ = evaluation;
+                container.extend([*expr]);
+            },
+            expr => expr.inputs_rev(container),
+        }
+    }
+
     /// Push the children of this node to the given container, in reverse order.
     /// This ensures the primary node responsible for the name is pushed last.
     ///
@@ -98,6 +132,8 @@ impl AExpr {
 
         match self {
             Element | Column(_) | Literal(_) | Len => {},
+            #[cfg(feature = "dtype-struct")]
+            StructField(_) => {},
             BinaryExpr { left, op: _, right } => {
                 container.extend([*right, *left]);
             },
@@ -158,6 +194,11 @@ impl AExpr {
                 evaluation,
                 variant: _,
             } => container.extend([*evaluation, *expr]),
+            #[cfg(feature = "dtype-struct")]
+            StructEval { expr, evaluation } => {
+                container.extend(evaluation.iter().rev().map(ExprIR::node));
+                container.extend([*expr]);
+            },
             Slice {
                 input,
                 offset,
@@ -172,6 +213,8 @@ impl AExpr {
         use AExpr::*;
         let input = match &mut self {
             Element | Column(_) | Literal(_) | Len => return self,
+            #[cfg(feature = "dtype-struct")]
+            StructField(_) => return self,
             Cast { expr, .. } => expr,
             Explode { expr, .. } => expr,
             BinaryExpr { left, right, .. } => {
@@ -198,10 +241,18 @@ impl AExpr {
             },
             Agg(a) => {
                 match a {
-                    IRAggExpr::Quantile { expr, quantile, .. } => {
+                    IRAggExpr::Quantile {
+                        expr,
+                        quantile,
+                        method: _,
+                    } => {
                         *expr = inputs[0];
                         *quantile = inputs[1];
                     },
+                    IRAggExpr::MinBy { input, by } | IRAggExpr::MaxBy { input, by } => {
+                        *input = inputs[0];
+                        *by = inputs[1];
+                    },
                     _ => {
                         a.set_input(inputs[0]);
                     },
@@ -236,6 +287,12 @@ impl AExpr {
                 _ = evaluation; // Intentional.
                 return self;
             },
+            #[cfg(feature = "dtype-struct")]
+            StructEval { expr, evaluation } => {
+                *expr = inputs[0];
+                _ = evaluation; // Intentional.
+                return self;
+            },
             Slice {
                 input,
                 offset,
@@ -282,6 +339,8 @@ impl AExpr {
         use AExpr::*;
         let input = match &mut self {
             Element | Column(_) | Literal(_) | Len => return self,
+            #[cfg(feature = "dtype-struct")]
+            StructField(_) => return self,
             Cast { expr, .. } => expr,
             Explode { expr, .. } => expr,
             BinaryExpr { left, right, .. } => {
@@ -312,6 +371,14 @@ impl AExpr {
                         *expr = inputs[0];
                         *quantile = inputs[1];
                     },
+                    IRAggExpr::MinBy { input, by } => {
+                        *input = inputs[0];
+                        *by = inputs[1];
+                    },
+                    IRAggExpr::MaxBy { input, by } => {
+                        *input = inputs[0];
+                        *by = inputs[1];
+                    },
                     _ => {
                         a.set_input(inputs[0]);
                     },
@@ -346,6 +413,15 @@ impl AExpr {
                 *evaluation = inputs[1];
                 return self;
             },
+            #[cfg(feature = "dtype-struct")]
+            StructEval { expr, evaluation } => {
+                assert_eq!(inputs.len(), evaluation.len() + 1);
+                *expr = inputs[0];
+                for (e, node) in evaluation.iter_mut().zip(inputs[1..].iter()) {
+                    e.set_node(*node);
+                }
+                return self;
+            },
             Slice {
                 input,
                 offset,
@@ -397,6 +473,8 @@ impl IRAggExpr {
         match self {
             Min { input, .. } => Single(*input),
             Max { input, .. } => Single(*input),
+            MinBy { input, by } => Many(vec![*input, *by]),
+            MaxBy { input, by } => Many(vec![*input, *by]),
             Median(input) => Single(*input),
             NUnique(input) => Single(*input),
             First(input) => Single(*input),
@@ -434,6 +512,10 @@ impl IRAggExpr {
             Std(input, _) => input,
             Var(input, _) => input,
             AggGroups(input) => input,
+
+            // Multi-input aggregations.
+            MinBy { .. } => unreachable!(),
+            MaxBy { .. } => unreachable!(),
         };
         *node = input;
     }
