@@ -3,12 +3,10 @@ use std::sync::Arc;
 
 use polars_core::config;
 use polars_core::error::{PolarsResult, polars_bail};
-use polars_core::frame::DataFrame;
 use polars_utils::arena::{Arena, Node};
 use polars_utils::pl_str::PlSmallStr;
 #[cfg(feature = "python")]
 use polars_utils::python_function::PythonObject;
-use polars_utils::row_counter::RowCounter;
 use polars_utils::slice_enum::Slice;
 use polars_utils::{format_pl_smallstr, unitvec};
 
@@ -248,9 +246,7 @@ pub(super) fn expand_datasets(
                                     .contains(&format_pl_smallstr!("{}_nc", row_index_name))
                             );
 
-                            statistics_df.clear_schema();
-
-                            unsafe { statistics_df.get_columns_mut() }.extend([
+                            unsafe { statistics_df.columns_mut() }.extend([
                                 IdxCa::from_vec(
                                     format_pl_smallstr!("{}_nc", row_index_name),
                                     vec![0],
@@ -333,64 +329,6 @@ pub(super) fn expand_datasets(
         }
 
         apply_scan_predicate_to_scan_ir(node, ir_arena, expr_arena)?;
-
-        let output_schema = ir_arena.get(node).schema(ir_arena);
-
-        let IR::Scan {
-            sources,
-            file_info: _,
-            hive_parts: _,
-            predicate,
-            predicate_file_skip_applied: _,
-            output_schema: _,
-            scan_type,
-            unified_scan_args,
-        } = ir_arena.get(node)
-        else {
-            unreachable!()
-        };
-
-        let df: DataFrame = if (sources.is_empty()
-            && !matches!(scan_type.as_ref(), FileScanIR::Anonymous { .. }))
-            || unified_scan_args
-                .pre_slice
-                .as_ref()
-                .is_some_and(|slice| slice.len() == 0)
-        {
-            if config::verbose() {
-                eprintln!("expand_datasets: scan IR replaced with empty DataFrameScan")
-            }
-
-            DataFrame::empty_with_schema(output_schema.as_ref())
-        } else if output_schema.is_empty()
-            && let Some((physical_rows, deleted_rows)) = unified_scan_args.row_count
-            && unified_scan_args.pre_slice.is_none()
-            && predicate.is_none()
-        {
-            let row_counter = RowCounter::new(physical_rows, deleted_rows);
-            row_counter.num_rows_idxsize()?;
-            let num_rows = row_counter.num_rows()?;
-
-            if config::verbose() {
-                eprintln!(
-                    "expand_datasets: scan IR replaced with 0-width DataFrameScan with height {} ({:?})",
-                    num_rows, &row_counter
-                )
-            }
-
-            DataFrame::empty_with_height(num_rows)
-        } else {
-            continue;
-        };
-
-        let schema = df.schema().clone();
-        let new_ir = IR::DataFrameScan {
-            df: Arc::new(df),
-            schema: schema.clone(),
-            output_schema: Some(schema),
-        };
-
-        ir_arena.replace(node, new_ir);
     }
 
     Ok(())
