@@ -26,12 +26,12 @@ use polars_compute::decimal::dec128_verify_prec_scale;
 use polars_core::datatypes::extension::get_extension_type_or_generic;
 use polars_core::schema::iceberg::IcebergSchema;
 use polars_core::utils::arrow::array::Array;
-use polars_core::utils::arrow::types::NativeType;
 use polars_core::utils::materialize_dyn_int;
 use polars_lazy::prelude::*;
 #[cfg(feature = "parquet")]
 use polars_parquet::write::StatisticsOptions;
 use polars_plan::dsl::ScanSources;
+use polars_utils::compression::{BrotliLevel, GzipLevel, ZstdLevel};
 use polars_utils::mmap::MemSlice;
 use polars_utils::pl_str::PlSmallStr;
 use polars_utils::total_ord::{TotalEq, TotalHash};
@@ -128,15 +128,19 @@ pub(crate) fn to_series(py: Python<'_>, s: PySeries) -> PyResult<Bound<'_, PyAny
     constructor.call1((s,))
 }
 
-impl<'py> FromPyObject<'py> for Wrap<PlSmallStr> {
-    fn extract_bound(ob: &Bound<'py, PyAny>) -> PyResult<Self> {
+impl<'a, 'py> FromPyObject<'a, 'py> for Wrap<PlSmallStr> {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
         Ok(Wrap((&*ob.extract::<PyBackedStr>()?).into()))
     }
 }
 
 #[cfg(feature = "csv")]
-impl<'py> FromPyObject<'py> for Wrap<NullValues> {
-    fn extract_bound(ob: &Bound<'py, PyAny>) -> PyResult<Self> {
+impl<'a, 'py> FromPyObject<'a, 'py> for Wrap<NullValues> {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
         if let Ok(s) = ob.extract::<PyBackedStr>() {
             Ok(Wrap(NullValues::AllColumnsSingle((&*s).into())))
         } else if let Ok(s) = ob.extract::<Vec<PyBackedStr>>() {
@@ -350,8 +354,10 @@ impl<'py> IntoPyObject<'py> for &Wrap<DataType> {
     }
 }
 
-impl<'py> FromPyObject<'py> for Wrap<Field> {
-    fn extract_bound(ob: &Bound<'py, PyAny>) -> PyResult<Self> {
+impl<'a, 'py> FromPyObject<'a, 'py> for Wrap<Field> {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
         let py = ob.py();
         let name = ob
             .getattr(intern!(py, "name"))?
@@ -364,8 +370,10 @@ impl<'py> FromPyObject<'py> for Wrap<Field> {
     }
 }
 
-impl<'py> FromPyObject<'py> for Wrap<DataType> {
-    fn extract_bound(ob: &Bound<'py, PyAny>) -> PyResult<Self> {
+impl<'a, 'py> FromPyObject<'a, 'py> for Wrap<DataType> {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
         let py = ob.py();
         let type_name = ob.get_type().qualname()?.to_string();
 
@@ -534,12 +542,14 @@ impl<'py> IntoPyObject<'py> for Wrap<TimeUnit> {
 }
 
 #[cfg(feature = "parquet")]
-impl<'py> FromPyObject<'py> for Wrap<StatisticsOptions> {
-    fn extract_bound(ob: &Bound<'py, PyAny>) -> PyResult<Self> {
+impl<'a, 'py> FromPyObject<'a, 'py> for Wrap<StatisticsOptions> {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
         let mut statistics = StatisticsOptions::empty();
 
-        let dict = ob.downcast::<PyDict>()?;
-        for (key, val) in dict {
+        let dict = ob.cast::<PyDict>()?;
+        for (key, val) in dict.iter() {
             let key = key.extract::<PyBackedStr>()?;
             let val = val.extract::<bool>()?;
 
@@ -560,17 +570,21 @@ impl<'py> FromPyObject<'py> for Wrap<StatisticsOptions> {
     }
 }
 
-impl<'py> FromPyObject<'py> for Wrap<Row<'static>> {
-    fn extract_bound(ob: &Bound<'py, PyAny>) -> PyResult<Self> {
+impl<'a, 'py> FromPyObject<'a, 'py> for Wrap<Row<'static>> {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
         let vals = ob.extract::<Vec<Wrap<AnyValue<'static>>>>()?;
         let vals = reinterpret_vec(vals);
         Ok(Wrap(Row(vals)))
     }
 }
 
-impl<'py> FromPyObject<'py> for Wrap<Schema> {
-    fn extract_bound(ob: &Bound<'py, PyAny>) -> PyResult<Self> {
-        let dict = ob.downcast::<PyDict>()?;
+impl<'a, 'py> FromPyObject<'a, 'py> for Wrap<Schema> {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
+        let dict = ob.cast::<PyDict>()?;
 
         Ok(Wrap(
             dict.iter()
@@ -585,8 +599,10 @@ impl<'py> FromPyObject<'py> for Wrap<Schema> {
     }
 }
 
-impl<'py> FromPyObject<'py> for Wrap<ArrowSchema> {
-    fn extract_bound(ob: &Bound<'py, PyAny>) -> PyResult<Self> {
+impl<'a, 'py> FromPyObject<'a, 'py> for Wrap<ArrowSchema> {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
         let py = ob.py();
 
         let pyarrow_schema_cls = py
@@ -628,9 +644,11 @@ impl<'py> FromPyObject<'py> for Wrap<ArrowSchema> {
     }
 }
 
-impl<'py> FromPyObject<'py> for Wrap<ScanSources> {
-    fn extract_bound(ob: &Bound<'py, PyAny>) -> PyResult<Self> {
-        let list = ob.downcast::<PyList>()?.to_owned();
+impl<'a, 'py> FromPyObject<'a, 'py> for Wrap<ScanSources> {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
+        let list = ob.cast::<PyList>()?.to_owned();
 
         if list.is_empty() {
             return Ok(Wrap(ScanSources::default()));
@@ -776,8 +794,10 @@ impl From<Py<PyAny>> for ObjectValue {
     }
 }
 
-impl<'py> FromPyObject<'py> for ObjectValue {
-    fn extract_bound(ob: &Bound<'py, PyAny>) -> PyResult<Self> {
+impl<'a, 'py> FromPyObject<'a, 'py> for ObjectValue {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
         Ok(ObjectValue {
             inner: ob.to_owned().unbind(),
         })
@@ -810,20 +830,29 @@ impl Default for ObjectValue {
     }
 }
 
-impl<'py, T: NativeType + FromPyObject<'py>> FromPyObject<'py> for Wrap<Vec<T>> {
-    fn extract_bound(obj: &Bound<'py, PyAny>) -> PyResult<Self> {
-        let seq = obj.downcast::<PySequence>()?;
+impl<'a, 'py, T> FromPyObject<'a, 'py> for Wrap<Vec<T>>
+where
+    T: FromPyObjectOwned<'py>,
+{
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
+        let seq = ob
+            .cast::<PySequence>()
+            .map_err(<PyErr as From<pyo3::CastError>>::from)?;
         let mut v = Vec::with_capacity(seq.len().unwrap_or(0));
         for item in seq.try_iter()? {
-            v.push(item?.extract::<T>()?);
+            v.push(item?.extract::<T>().map_err(Into::into)?);
         }
         Ok(Wrap(v))
     }
 }
 
 #[cfg(feature = "asof_join")]
-impl<'py> FromPyObject<'py> for Wrap<AsofStrategy> {
-    fn extract_bound(ob: &Bound<'py, PyAny>) -> PyResult<Self> {
+impl<'a, 'py> FromPyObject<'a, 'py> for Wrap<AsofStrategy> {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
         let parsed = match &*(ob.extract::<PyBackedStr>()?) {
             "backward" => AsofStrategy::Backward,
             "forward" => AsofStrategy::Forward,
@@ -838,8 +867,10 @@ impl<'py> FromPyObject<'py> for Wrap<AsofStrategy> {
     }
 }
 
-impl<'py> FromPyObject<'py> for Wrap<InterpolationMethod> {
-    fn extract_bound(ob: &Bound<'py, PyAny>) -> PyResult<Self> {
+impl<'a, 'py> FromPyObject<'a, 'py> for Wrap<InterpolationMethod> {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
         let parsed = match &*(ob.extract::<PyBackedStr>()?) {
             "linear" => InterpolationMethod::Linear,
             "nearest" => InterpolationMethod::Nearest,
@@ -854,8 +885,10 @@ impl<'py> FromPyObject<'py> for Wrap<InterpolationMethod> {
 }
 
 #[cfg(feature = "avro")]
-impl<'py> FromPyObject<'py> for Wrap<Option<AvroCompression>> {
-    fn extract_bound(ob: &Bound<'py, PyAny>) -> PyResult<Self> {
+impl<'a, 'py> FromPyObject<'a, 'py> for Wrap<Option<AvroCompression>> {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
         let parsed = match &*ob.extract::<PyBackedStr>()? {
             "uncompressed" => None,
             "snappy" => Some(AvroCompression::Snappy),
@@ -870,8 +903,10 @@ impl<'py> FromPyObject<'py> for Wrap<Option<AvroCompression>> {
     }
 }
 
-impl<'py> FromPyObject<'py> for Wrap<StartBy> {
-    fn extract_bound(ob: &Bound<'py, PyAny>) -> PyResult<Self> {
+impl<'a, 'py> FromPyObject<'a, 'py> for Wrap<StartBy> {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
         let parsed = match &*ob.extract::<PyBackedStr>()? {
             "window" => StartBy::WindowBound,
             "datapoint" => StartBy::DataPoint,
@@ -892,8 +927,10 @@ impl<'py> FromPyObject<'py> for Wrap<StartBy> {
     }
 }
 
-impl<'py> FromPyObject<'py> for Wrap<ClosedWindow> {
-    fn extract_bound(ob: &Bound<'py, PyAny>) -> PyResult<Self> {
+impl<'a, 'py> FromPyObject<'a, 'py> for Wrap<ClosedWindow> {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
         let parsed = match &*ob.extract::<PyBackedStr>()? {
             "left" => ClosedWindow::Left,
             "right" => ClosedWindow::Right,
@@ -909,8 +946,10 @@ impl<'py> FromPyObject<'py> for Wrap<ClosedWindow> {
     }
 }
 
-impl<'py> FromPyObject<'py> for Wrap<RoundMode> {
-    fn extract_bound(ob: &Bound<'py, PyAny>) -> PyResult<Self> {
+impl<'a, 'py> FromPyObject<'a, 'py> for Wrap<RoundMode> {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
         let parsed = match &*ob.extract::<PyBackedStr>()? {
             "half_to_even" => RoundMode::HalfToEven,
             "half_away_from_zero" => RoundMode::HalfAwayFromZero,
@@ -925,8 +964,10 @@ impl<'py> FromPyObject<'py> for Wrap<RoundMode> {
 }
 
 #[cfg(feature = "csv")]
-impl<'py> FromPyObject<'py> for Wrap<CsvEncoding> {
-    fn extract_bound(ob: &Bound<'py, PyAny>) -> PyResult<Self> {
+impl<'a, 'py> FromPyObject<'a, 'py> for Wrap<CsvEncoding> {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
         let parsed = match &*ob.extract::<PyBackedStr>()? {
             "utf8" => CsvEncoding::Utf8,
             "utf8-lossy" => CsvEncoding::LossyUtf8,
@@ -941,8 +982,10 @@ impl<'py> FromPyObject<'py> for Wrap<CsvEncoding> {
 }
 
 #[cfg(feature = "ipc")]
-impl<'py> FromPyObject<'py> for Wrap<Option<IpcCompression>> {
-    fn extract_bound(ob: &Bound<'py, PyAny>) -> PyResult<Self> {
+impl<'a, 'py> FromPyObject<'a, 'py> for Wrap<Option<IpcCompression>> {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
         let parsed = match &*ob.extract::<PyBackedStr>()? {
             "uncompressed" => None,
             "lz4" => Some(IpcCompression::LZ4),
@@ -957,8 +1000,10 @@ impl<'py> FromPyObject<'py> for Wrap<Option<IpcCompression>> {
     }
 }
 
-impl<'py> FromPyObject<'py> for Wrap<JoinType> {
-    fn extract_bound(ob: &Bound<'py, PyAny>) -> PyResult<Self> {
+impl<'a, 'py> FromPyObject<'a, 'py> for Wrap<JoinType> {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
         let parsed = match &*ob.extract::<PyBackedStr>()? {
             "inner" => JoinType::Inner,
             "left" => JoinType::Left,
@@ -978,8 +1023,10 @@ impl<'py> FromPyObject<'py> for Wrap<JoinType> {
     }
 }
 
-impl<'py> FromPyObject<'py> for Wrap<Label> {
-    fn extract_bound(ob: &Bound<'py, PyAny>) -> PyResult<Self> {
+impl<'a, 'py> FromPyObject<'a, 'py> for Wrap<Label> {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
         let parsed = match &*ob.extract::<PyBackedStr>()? {
             "left" => Label::Left,
             "right" => Label::Right,
@@ -994,8 +1041,10 @@ impl<'py> FromPyObject<'py> for Wrap<Label> {
     }
 }
 
-impl<'py> FromPyObject<'py> for Wrap<ListToStructWidthStrategy> {
-    fn extract_bound(ob: &Bound<'py, PyAny>) -> PyResult<Self> {
+impl<'a, 'py> FromPyObject<'a, 'py> for Wrap<ListToStructWidthStrategy> {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
         let parsed = match &*ob.extract::<PyBackedStr>()? {
             "first_non_null" => ListToStructWidthStrategy::FirstNonNull,
             "max_width" => ListToStructWidthStrategy::MaxWidth,
@@ -1009,8 +1058,10 @@ impl<'py> FromPyObject<'py> for Wrap<ListToStructWidthStrategy> {
     }
 }
 
-impl<'py> FromPyObject<'py> for Wrap<NonExistent> {
-    fn extract_bound(ob: &Bound<'py, PyAny>) -> PyResult<Self> {
+impl<'a, 'py> FromPyObject<'a, 'py> for Wrap<NonExistent> {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
         let parsed = match &*ob.extract::<PyBackedStr>()? {
             "null" => NonExistent::Null,
             "raise" => NonExistent::Raise,
@@ -1024,8 +1075,10 @@ impl<'py> FromPyObject<'py> for Wrap<NonExistent> {
     }
 }
 
-impl<'py> FromPyObject<'py> for Wrap<NullBehavior> {
-    fn extract_bound(ob: &Bound<'py, PyAny>) -> PyResult<Self> {
+impl<'a, 'py> FromPyObject<'a, 'py> for Wrap<NullBehavior> {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
         let parsed = match &*ob.extract::<PyBackedStr>()? {
             "drop" => NullBehavior::Drop,
             "ignore" => NullBehavior::Ignore,
@@ -1039,8 +1092,10 @@ impl<'py> FromPyObject<'py> for Wrap<NullBehavior> {
     }
 }
 
-impl<'py> FromPyObject<'py> for Wrap<NullStrategy> {
-    fn extract_bound(ob: &Bound<'py, PyAny>) -> PyResult<Self> {
+impl<'a, 'py> FromPyObject<'a, 'py> for Wrap<NullStrategy> {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
         let parsed = match &*ob.extract::<PyBackedStr>()? {
             "ignore" => NullStrategy::Ignore,
             "propagate" => NullStrategy::Propagate,
@@ -1055,8 +1110,10 @@ impl<'py> FromPyObject<'py> for Wrap<NullStrategy> {
 }
 
 #[cfg(feature = "parquet")]
-impl<'py> FromPyObject<'py> for Wrap<ParallelStrategy> {
-    fn extract_bound(ob: &Bound<'py, PyAny>) -> PyResult<Self> {
+impl<'a, 'py> FromPyObject<'a, 'py> for Wrap<ParallelStrategy> {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
         let parsed = match &*ob.extract::<PyBackedStr>()? {
             "auto" => ParallelStrategy::Auto,
             "columns" => ParallelStrategy::Columns,
@@ -1073,8 +1130,10 @@ impl<'py> FromPyObject<'py> for Wrap<ParallelStrategy> {
     }
 }
 
-impl<'py> FromPyObject<'py> for Wrap<IndexOrder> {
-    fn extract_bound(ob: &Bound<'py, PyAny>) -> PyResult<Self> {
+impl<'a, 'py> FromPyObject<'a, 'py> for Wrap<IndexOrder> {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
         let parsed = match &*ob.extract::<PyBackedStr>()? {
             "fortran" => IndexOrder::Fortran,
             "c" => IndexOrder::C,
@@ -1088,8 +1147,10 @@ impl<'py> FromPyObject<'py> for Wrap<IndexOrder> {
     }
 }
 
-impl<'py> FromPyObject<'py> for Wrap<QuantileMethod> {
-    fn extract_bound(ob: &Bound<'py, PyAny>) -> PyResult<Self> {
+impl<'a, 'py> FromPyObject<'a, 'py> for Wrap<QuantileMethod> {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
         let parsed = match &*ob.extract::<PyBackedStr>()? {
             "lower" => QuantileMethod::Lower,
             "higher" => QuantileMethod::Higher,
@@ -1107,8 +1168,10 @@ impl<'py> FromPyObject<'py> for Wrap<QuantileMethod> {
     }
 }
 
-impl<'py> FromPyObject<'py> for Wrap<RankMethod> {
-    fn extract_bound(ob: &Bound<'py, PyAny>) -> PyResult<Self> {
+impl<'a, 'py> FromPyObject<'a, 'py> for Wrap<RankMethod> {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
         let parsed = match &*ob.extract::<PyBackedStr>()? {
             "min" => RankMethod::Min,
             "max" => RankMethod::Max,
@@ -1126,8 +1189,10 @@ impl<'py> FromPyObject<'py> for Wrap<RankMethod> {
     }
 }
 
-impl<'py> FromPyObject<'py> for Wrap<RollingRankMethod> {
-    fn extract_bound(ob: &Bound<'py, PyAny>) -> PyResult<Self> {
+impl<'a, 'py> FromPyObject<'a, 'py> for Wrap<RollingRankMethod> {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
         let parsed = match &*ob.extract::<PyBackedStr>()? {
             "min" => RollingRankMethod::Min,
             "max" => RollingRankMethod::Max,
@@ -1144,8 +1209,10 @@ impl<'py> FromPyObject<'py> for Wrap<RollingRankMethod> {
     }
 }
 
-impl<'py> FromPyObject<'py> for Wrap<Roll> {
-    fn extract_bound(ob: &Bound<'py, PyAny>) -> PyResult<Self> {
+impl<'a, 'py> FromPyObject<'a, 'py> for Wrap<Roll> {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
         let parsed = match &*ob.extract::<PyBackedStr>()? {
             "raise" => Roll::Raise,
             "forward" => Roll::Forward,
@@ -1160,8 +1227,10 @@ impl<'py> FromPyObject<'py> for Wrap<Roll> {
     }
 }
 
-impl<'py> FromPyObject<'py> for Wrap<TimeUnit> {
-    fn extract_bound(ob: &Bound<'py, PyAny>) -> PyResult<Self> {
+impl<'a, 'py> FromPyObject<'a, 'py> for Wrap<TimeUnit> {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
         let parsed = match &*ob.extract::<PyBackedStr>()? {
             "ns" => TimeUnit::Nanoseconds,
             "us" => TimeUnit::Microseconds,
@@ -1176,8 +1245,10 @@ impl<'py> FromPyObject<'py> for Wrap<TimeUnit> {
     }
 }
 
-impl<'py> FromPyObject<'py> for Wrap<UniqueKeepStrategy> {
-    fn extract_bound(ob: &Bound<'py, PyAny>) -> PyResult<Self> {
+impl<'a, 'py> FromPyObject<'a, 'py> for Wrap<UniqueKeepStrategy> {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
         let parsed = match &*ob.extract::<PyBackedStr>()? {
             "first" => UniqueKeepStrategy::First,
             "last" => UniqueKeepStrategy::Last,
@@ -1194,8 +1265,10 @@ impl<'py> FromPyObject<'py> for Wrap<UniqueKeepStrategy> {
 }
 
 #[cfg(feature = "search_sorted")]
-impl<'py> FromPyObject<'py> for Wrap<SearchSortedSide> {
-    fn extract_bound(ob: &Bound<'py, PyAny>) -> PyResult<Self> {
+impl<'a, 'py> FromPyObject<'a, 'py> for Wrap<SearchSortedSide> {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
         let parsed = match &*ob.extract::<PyBackedStr>()? {
             "any" => SearchSortedSide::Any,
             "left" => SearchSortedSide::Left,
@@ -1210,8 +1283,10 @@ impl<'py> FromPyObject<'py> for Wrap<SearchSortedSide> {
     }
 }
 
-impl<'py> FromPyObject<'py> for Wrap<ClosedInterval> {
-    fn extract_bound(ob: &Bound<'py, PyAny>) -> PyResult<Self> {
+impl<'a, 'py> FromPyObject<'a, 'py> for Wrap<ClosedInterval> {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
         let parsed = match &*ob.extract::<PyBackedStr>()? {
             "both" => ClosedInterval::Both,
             "left" => ClosedInterval::Left,
@@ -1227,8 +1302,10 @@ impl<'py> FromPyObject<'py> for Wrap<ClosedInterval> {
     }
 }
 
-impl<'py> FromPyObject<'py> for Wrap<WindowMapping> {
-    fn extract_bound(ob: &Bound<'py, PyAny>) -> PyResult<Self> {
+impl<'a, 'py> FromPyObject<'a, 'py> for Wrap<WindowMapping> {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
         let parsed = match &*ob.extract::<PyBackedStr>()? {
             "group_to_rows" => WindowMapping::GroupsToRows,
             "join" => WindowMapping::Join,
@@ -1243,8 +1320,10 @@ impl<'py> FromPyObject<'py> for Wrap<WindowMapping> {
     }
 }
 
-impl<'py> FromPyObject<'py> for Wrap<JoinValidation> {
-    fn extract_bound(ob: &Bound<'py, PyAny>) -> PyResult<Self> {
+impl<'a, 'py> FromPyObject<'a, 'py> for Wrap<JoinValidation> {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
         let parsed = match &*ob.extract::<PyBackedStr>()? {
             "1:1" => JoinValidation::OneToOne,
             "1:m" => JoinValidation::OneToMany,
@@ -1260,8 +1339,10 @@ impl<'py> FromPyObject<'py> for Wrap<JoinValidation> {
     }
 }
 
-impl<'py> FromPyObject<'py> for Wrap<MaintainOrderJoin> {
-    fn extract_bound(ob: &Bound<'py, PyAny>) -> PyResult<Self> {
+impl<'a, 'py> FromPyObject<'a, 'py> for Wrap<MaintainOrderJoin> {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
         let parsed = match &*ob.extract::<PyBackedStr>()? {
             "none" => MaintainOrderJoin::None,
             "left" => MaintainOrderJoin::Left,
@@ -1279,8 +1360,10 @@ impl<'py> FromPyObject<'py> for Wrap<MaintainOrderJoin> {
 }
 
 #[cfg(feature = "csv")]
-impl<'py> FromPyObject<'py> for Wrap<QuoteStyle> {
-    fn extract_bound(ob: &Bound<'py, PyAny>) -> PyResult<Self> {
+impl<'a, 'py> FromPyObject<'a, 'py> for Wrap<QuoteStyle> {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
         let parsed = match &*ob.extract::<PyBackedStr>()? {
             "always" => QuoteStyle::Always,
             "necessary" => QuoteStyle::Necessary,
@@ -1307,8 +1390,10 @@ pub(crate) fn parse_cloud_options(
 }
 
 #[cfg(feature = "list_sets")]
-impl<'py> FromPyObject<'py> for Wrap<SetOperation> {
-    fn extract_bound(ob: &Bound<'py, PyAny>) -> PyResult<Self> {
+impl<'a, 'py> FromPyObject<'a, 'py> for Wrap<SetOperation> {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
         let parsed = match &*ob.extract::<PyBackedStr>()? {
             "union" => SetOperation::Union,
             "difference" => SetOperation::Difference,
@@ -1325,8 +1410,10 @@ impl<'py> FromPyObject<'py> for Wrap<SetOperation> {
 }
 
 // Conversion from ScanCastOptions class from the Python side.
-impl<'py> FromPyObject<'py> for Wrap<CastColumnsPolicy> {
-    fn extract_bound(ob: &Bound<'py, PyAny>) -> PyResult<Self> {
+impl<'a, 'py> FromPyObject<'a, 'py> for Wrap<CastColumnsPolicy> {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
         if ob.is_none() {
             // Initialize the default ScanCastOptions from Python.
             static DEFAULT: PyOnceLock<Wrap<CastColumnsPolicy>> = PyOnceLock::new();
@@ -1339,7 +1426,7 @@ impl<'py> FromPyObject<'py> for Wrap<CastColumnsPolicy> {
                     .call_method0("_default")
                     .unwrap();
 
-                let out = Self::extract_bound(&ob)?;
+                let out = Self::extract(ob.as_borrowed())?;
 
                 // The default policy should match ERROR_ON_MISMATCH (but this can change).
                 debug_assert_eq!(&out.0, &CastColumnsPolicy::ERROR_ON_MISMATCH);
@@ -1555,8 +1642,10 @@ where
 #[derive(Debug, Copy, Clone)]
 pub struct PyCompatLevel(pub CompatLevel);
 
-impl<'py> FromPyObject<'py> for PyCompatLevel {
-    fn extract_bound(ob: &Bound<'py, PyAny>) -> PyResult<Self> {
+impl<'a, 'py> FromPyObject<'a, 'py> for PyCompatLevel {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
         Ok(PyCompatLevel(if let Ok(level) = ob.extract::<u16>() {
             if let Ok(compat_level) = CompatLevel::with_level(level) {
                 compat_level
@@ -1578,8 +1667,10 @@ impl<'py> FromPyObject<'py> for PyCompatLevel {
 }
 
 #[cfg(feature = "string_normalize")]
-impl<'py> FromPyObject<'py> for Wrap<UnicodeForm> {
-    fn extract_bound(ob: &Bound<'py, PyAny>) -> PyResult<Self> {
+impl<'a, 'py> FromPyObject<'a, 'py> for Wrap<UnicodeForm> {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
         let parsed = match &*ob.extract::<PyBackedStr>()? {
             "NFC" => UnicodeForm::NFC,
             "NFKC" => UnicodeForm::NFKC,
@@ -1596,15 +1687,17 @@ impl<'py> FromPyObject<'py> for Wrap<UnicodeForm> {
 }
 
 #[cfg(feature = "parquet")]
-impl<'py> FromPyObject<'py> for Wrap<Option<KeyValueMetadata>> {
-    fn extract_bound(ob: &Bound<'py, PyAny>) -> PyResult<Self> {
+impl<'a, 'py> FromPyObject<'a, 'py> for Wrap<Option<KeyValueMetadata>> {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
         #[derive(FromPyObject)]
         enum Metadata {
             Static(Vec<(String, String)>),
             Dynamic(Py<PyAny>),
         }
 
-        let metadata = Option::<Metadata>::extract_bound(ob)?;
+        let metadata = Option::<Metadata>::extract(ob)?;
         let key_value_metadata = metadata.map(|x| match x {
             Metadata::Static(kv) => KeyValueMetadata::from_static(kv),
             Metadata::Dynamic(func) => KeyValueMetadata::from_py_function(func),
@@ -1613,9 +1706,11 @@ impl<'py> FromPyObject<'py> for Wrap<Option<KeyValueMetadata>> {
     }
 }
 
-impl<'py> FromPyObject<'py> for Wrap<Option<TimeZone>> {
-    fn extract_bound(ob: &Bound<'py, PyAny>) -> PyResult<Self> {
-        let tz = Option::<Wrap<PlSmallStr>>::extract_bound(ob)?;
+impl<'a, 'py> FromPyObject<'a, 'py> for Wrap<Option<TimeZone>> {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
+        let tz = Option::<Wrap<PlSmallStr>>::extract(ob)?;
 
         let tz = tz.map(|x| x.0);
 
@@ -1623,8 +1718,10 @@ impl<'py> FromPyObject<'py> for Wrap<Option<TimeZone>> {
     }
 }
 
-impl<'py> FromPyObject<'py> for Wrap<UpcastOrForbid> {
-    fn extract_bound(ob: &Bound<'py, PyAny>) -> PyResult<Self> {
+impl<'a, 'py> FromPyObject<'a, 'py> for Wrap<UpcastOrForbid> {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
         let parsed = match &*ob.extract::<PyBackedStr>()? {
             "upcast" => UpcastOrForbid::Upcast,
             "forbid" => UpcastOrForbid::Forbid,
@@ -1638,8 +1735,10 @@ impl<'py> FromPyObject<'py> for Wrap<UpcastOrForbid> {
     }
 }
 
-impl<'py> FromPyObject<'py> for Wrap<ExtraColumnsPolicy> {
-    fn extract_bound(ob: &Bound<'py, PyAny>) -> PyResult<Self> {
+impl<'a, 'py> FromPyObject<'a, 'py> for Wrap<ExtraColumnsPolicy> {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
         let parsed = match &*ob.extract::<PyBackedStr>()? {
             "ignore" => ExtraColumnsPolicy::Ignore,
             "raise" => ExtraColumnsPolicy::Raise,
@@ -1653,8 +1752,10 @@ impl<'py> FromPyObject<'py> for Wrap<ExtraColumnsPolicy> {
     }
 }
 
-impl<'py> FromPyObject<'py> for Wrap<MissingColumnsPolicy> {
-    fn extract_bound(ob: &Bound<'py, PyAny>) -> PyResult<Self> {
+impl<'a, 'py> FromPyObject<'a, 'py> for Wrap<MissingColumnsPolicy> {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
         let parsed = match &*ob.extract::<PyBackedStr>()? {
             "insert" => MissingColumnsPolicy::Insert,
             "raise" => MissingColumnsPolicy::Raise,
@@ -1668,8 +1769,10 @@ impl<'py> FromPyObject<'py> for Wrap<MissingColumnsPolicy> {
     }
 }
 
-impl<'py> FromPyObject<'py> for Wrap<MissingColumnsPolicyOrExpr> {
-    fn extract_bound(ob: &Bound<'py, PyAny>) -> PyResult<Self> {
+impl<'a, 'py> FromPyObject<'a, 'py> for Wrap<MissingColumnsPolicyOrExpr> {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
         if let Ok(pyexpr) = ob.extract::<PyExpr>() {
             return Ok(Wrap(MissingColumnsPolicyOrExpr::InsertWith(pyexpr.inner)));
         }
@@ -1687,8 +1790,10 @@ impl<'py> FromPyObject<'py> for Wrap<MissingColumnsPolicyOrExpr> {
     }
 }
 
-impl<'py> FromPyObject<'py> for Wrap<ColumnMapping> {
-    fn extract_bound(ob: &Bound<'py, PyAny>) -> PyResult<Self> {
+impl<'a, 'py> FromPyObject<'a, 'py> for Wrap<ColumnMapping> {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
         let (column_mapping_type, ob): (PyBackedStr, Bound<'_, PyAny>) = ob.extract()?;
 
         Ok(Wrap(match &*column_mapping_type {
@@ -1708,8 +1813,10 @@ impl<'py> FromPyObject<'py> for Wrap<ColumnMapping> {
     }
 }
 
-impl<'py> FromPyObject<'py> for Wrap<DeletionFilesList> {
-    fn extract_bound(ob: &Bound<'py, PyAny>) -> PyResult<Self> {
+impl<'a, 'py> FromPyObject<'a, 'py> for Wrap<DeletionFilesList> {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
         let (deletion_file_type, ob): (PyBackedStr, Bound<'_, PyAny>) = ob.extract()?;
 
         Ok(Wrap(match &*deletion_file_type {
@@ -1752,8 +1859,10 @@ impl<'py> FromPyObject<'py> for Wrap<DeletionFilesList> {
     }
 }
 
-impl<'py> FromPyObject<'py> for Wrap<DefaultFieldValues> {
-    fn extract_bound(ob: &Bound<'py, PyAny>) -> PyResult<Self> {
+impl<'a, 'py> FromPyObject<'a, 'py> for Wrap<DefaultFieldValues> {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
         let (default_values_type, ob): (PyBackedStr, Bound<'_, PyAny>) = ob.extract()?;
 
         Ok(Wrap(match &*default_values_type {
@@ -1793,8 +1902,10 @@ impl<'py> FromPyObject<'py> for Wrap<DefaultFieldValues> {
     }
 }
 
-impl<'py> FromPyObject<'py> for Wrap<PlPath> {
-    fn extract_bound(ob: &Bound<'py, PyAny>) -> PyResult<Self> {
+impl<'a, 'py> FromPyObject<'a, 'py> for Wrap<PlPath> {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
         if let Ok(path) = ob.extract::<PyBackedStr>() {
             Ok(Wrap(PlPath::new(&path)))
         } else if let Ok(path) = ob.extract::<std::path::PathBuf>() {
