@@ -75,14 +75,20 @@ impl PyFileLikeObject {
                 .call_method(py, "read", (), None)
                 .expect("no read method found");
 
-            if let Ok(b) = bytes.downcast_bound::<PyBytes>(py) {
-                return MemSlice::from_arc(b.as_bytes(), Arc::new(bytes.clone_ref(py)));
+            if let Ok(b) = bytes.cast_bound::<PyBytes>(py) {
+                // SAFETY: we keep the underlying python object alive.
+                let slice = b.as_bytes();
+                let arc = Arc::new(bytes.clone_ref(py));
+                return unsafe { MemSlice::from_arc(slice, arc) };
             }
 
-            if let Ok(b) = bytes.downcast_bound::<PyString>(py) {
+            if let Ok(b) = bytes.cast_bound::<PyString>(py) {
                 return match b.to_cow().expect("PyString is not valid UTF-8") {
                     Cow::Borrowed(v) => {
-                        MemSlice::from_arc(v.as_bytes(), Arc::new(bytes.clone_ref(py)))
+                        // SAFETY: we keep the underlying python object alive.
+                        let slice = v.as_bytes();
+                        let arc = Arc::new(bytes.clone_ref(py));
+                        unsafe { MemSlice::from_arc(slice, arc) }
                     },
                     Cow::Owned(v) => MemSlice::from_vec(v.into_bytes()),
                 };
@@ -158,13 +164,13 @@ impl Read for PyFileLikeObject {
                 .call_method(py, "read", (buf.len(),), None)
                 .map_err(pyerr_to_io_err)?;
 
-            let opt_bytes = bytes.downcast_bound::<PyBytes>(py);
+            let opt_bytes = bytes.cast_bound::<PyBytes>(py);
 
             if let Ok(bytes) = opt_bytes {
                 buf.write_all(bytes.as_bytes())?;
 
                 bytes.len().map_err(pyerr_to_io_err)
-            } else if let Ok(s) = bytes.downcast_bound::<PyString>(py) {
+            } else if let Ok(s) = bytes.cast_bound::<PyString>(py) {
                 let s = s.to_cow().map_err(pyerr_to_io_err)?;
                 buf.write_all(s.as_bytes())?;
                 Ok(s.len())
@@ -411,12 +417,12 @@ pub(crate) fn get_python_scan_source_input(
         let py_f = read_if_bytesio(py_f);
 
         // If the pyobject is a `bytes` class
-        if let Ok(b) = py_f.downcast::<PyBytes>() {
-            return Ok(PythonScanSourceInput::Buffer(MemSlice::from_arc(
-                b.as_bytes(),
-                // We want to specifically keep alive the PyBytes object.
-                Arc::new(b.clone().unbind()),
-            )));
+        if let Ok(b) = py_f.cast::<PyBytes>() {
+            // SAFETY: we keep the underlying python object alive.
+            let slice = b.as_bytes();
+            let arc = Arc::new(b.clone().unbind());
+            let memslice = unsafe { MemSlice::from_arc(slice, arc) };
+            return Ok(PythonScanSourceInput::Buffer(memslice));
         }
 
         if let Ok(s) = py_f.extract::<Cow<str>>() {
@@ -490,14 +496,12 @@ pub(crate) fn get_mmap_bytes_reader_and_path(
     let py_f = read_if_bytesio(py_f.clone());
 
     // bytes object
-    if let Ok(bytes) = py_f.downcast::<PyBytes>() {
-        Ok((
-            Box::new(Cursor::new(MemSlice::from_arc(
-                bytes.as_bytes(),
-                Arc::new(py_f.clone().unbind()),
-            ))),
-            None,
-        ))
+    if let Ok(bytes) = py_f.cast::<PyBytes>() {
+        // SAFETY: we keep the underlying python object alive.
+        let slice = bytes.as_bytes();
+        let arc = Arc::new(bytes.clone().unbind());
+        let memslice = unsafe { MemSlice::from_arc(slice, arc) };
+        Ok((Box::new(Cursor::new(memslice)), None))
     }
     // string so read file
     else {
