@@ -4,19 +4,20 @@ use polars::prelude::sink2::FileProviderReturn;
 use polars::prelude::sync_on_close::SyncOnCloseType;
 use polars::prelude::{PartitionTargetCallbackResult, PlPath, SpecialEq};
 use pyo3::exceptions::PyValueError;
+use pyo3::prelude::*;
 use pyo3::pybacked::PyBackedStr;
-use pyo3::types::PyAnyMethods;
-use pyo3::{Bound, FromPyObject, PyAny, PyResult, Python};
 
 use crate::prelude::Wrap;
 
-impl<'py> FromPyObject<'py> for Wrap<polars_plan::dsl::SinkTarget> {
-    fn extract_bound(ob: &Bound<'py, PyAny>) -> PyResult<Self> {
+impl<'a, 'py> FromPyObject<'a, 'py> for Wrap<polars_plan::dsl::SinkTarget> {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
         if let Ok(v) = ob.extract::<PyBackedStr>() {
             Ok(Wrap(polars::prelude::SinkTarget::Path(PlPath::new(&v))))
         } else {
             let writer = Python::attach(|py| {
-                let py_f = ob.clone();
+                let py_f = ob.to_owned();
                 PyResult::Ok(
                     crate::file::try_get_pyfile(py, py_f, true)?
                         .0
@@ -31,8 +32,10 @@ impl<'py> FromPyObject<'py> for Wrap<polars_plan::dsl::SinkTarget> {
     }
 }
 
-impl<'py> FromPyObject<'py> for Wrap<PartitionTargetCallbackResult> {
-    fn extract_bound(ob: &Bound<'py, PyAny>) -> PyResult<Self> {
+impl<'a, 'py> FromPyObject<'a, 'py> for Wrap<PartitionTargetCallbackResult> {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
         if let Ok(v) = ob.extract::<PyBackedStr>() {
             Ok(Wrap(polars::prelude::PartitionTargetCallbackResult::Str(
                 v.to_string(),
@@ -43,7 +46,7 @@ impl<'py> FromPyObject<'py> for Wrap<PartitionTargetCallbackResult> {
             )))
         } else {
             let writer = Python::attach(|py| {
-                let py_f = ob.clone();
+                let py_f = ob.to_owned();
                 PyResult::Ok(
                     crate::file::try_get_pyfile(py, py_f, true)?
                         .0
@@ -60,8 +63,10 @@ impl<'py> FromPyObject<'py> for Wrap<PartitionTargetCallbackResult> {
     }
 }
 
-impl<'py> FromPyObject<'py> for Wrap<FileProviderReturn> {
-    fn extract_bound(ob: &Bound<'py, PyAny>) -> PyResult<Self> {
+impl<'a, 'py> FromPyObject<'a, 'py> for Wrap<FileProviderReturn> {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
         if let Ok(v) = ob.extract::<PyBackedStr>() {
             Ok(Wrap(FileProviderReturn::Path(v.to_string())))
         } else if let Ok(v) = ob.extract::<std::path::PathBuf>() {
@@ -71,7 +76,7 @@ impl<'py> FromPyObject<'py> for Wrap<FileProviderReturn> {
         } else {
             let py = ob.py();
 
-            let writeable = crate::file::try_get_pyfile(py, ob.clone(), true)?
+            let writeable = crate::file::try_get_pyfile(py, ob.to_owned(), true)?
                 .0
                 .into_writeable();
 
@@ -80,8 +85,10 @@ impl<'py> FromPyObject<'py> for Wrap<FileProviderReturn> {
     }
 }
 
-impl<'py> FromPyObject<'py> for Wrap<SyncOnCloseType> {
-    fn extract_bound(ob: &Bound<'py, PyAny>) -> PyResult<Self> {
+impl<'a, 'py> FromPyObject<'a, 'py> for Wrap<SyncOnCloseType> {
+    type Error = PyErr;
+
+    fn extract(ob: Borrowed<'a, 'py, PyAny>) -> PyResult<Self> {
         let parsed = match &*ob.extract::<PyBackedStr>()? {
             "none" => SyncOnCloseType::None,
             "data" => SyncOnCloseType::Data,
