@@ -1,12 +1,13 @@
 use std::borrow::Cow;
 
-use arrow::array::Array;
+use arrow::array::{Array, PrimitiveArray};
 use arrow::bitmap::BitmapBuilder;
+use arrow::buffer::Buffer;
+use arrow::storage::SharedStorage;
 use arrow::types::NativeType;
 use num_traits::AsPrimitive;
 use numpy::{Element, PyArray1, PyArrayMethods, PyUntypedArrayMethods};
 use polars::prelude::*;
-use polars_core::utils::CustomIterTools;
 use pyo3::exceptions::{PyTypeError, PyValueError};
 use pyo3::prelude::*;
 
@@ -25,7 +26,10 @@ macro_rules! init_method {
         impl PySeries {
             #[staticmethod]
             fn $name(name: &str, array: &Bound<PyArray1<$type>>, _strict: bool) -> Self {
-                mmap_numpy_array(name, array)
+                let arr = numpy_array_to_arrow(array);
+                Series::from_arrow(name.into(), arr.to_boxed())
+                    .unwrap()
+                    .into()
             }
         }
     };
@@ -40,13 +44,31 @@ init_method!(new_u16, u16);
 init_method!(new_u32, u32);
 init_method!(new_u64, u64);
 
-fn mmap_numpy_array<T: Element + NativeType>(name: &str, array: &Bound<PyArray1<T>>) -> PySeries {
-    let vals = unsafe { array.as_slice().unwrap() };
+fn numpy_array_to_arrow<T: Element + NativeType>(array: &Bound<PyArray1<T>>) -> PrimitiveArray<T> {
+    let owner = Arc::new(array.clone().unbind());
+    let ro = array.readonly();
+    let vals = ro.as_slice().unwrap();
+    unsafe {
+        let storage = SharedStorage::from_slice_with_owner(vals, owner);
+        let buffer = Buffer::from_storage(storage);
+        PrimitiveArray::new_unchecked(T::PRIMITIVE.into(), buffer, None)
+    }
+}
 
-    let arr = unsafe { arrow::ffi::mmap::slice_and_owner(vals, array.clone().unbind()) };
-    Series::from_arrow(name.into(), arr.to_boxed())
-        .unwrap()
-        .into()
+#[cfg(feature = "object")]
+pub fn series_from_objects(py: Python<'_>, name: PlSmallStr, objects: Vec<ObjectValue>) -> Series {
+    let mut validity = BitmapBuilder::with_capacity(objects.len());
+    for v in &objects {
+        let is_valid = !v.inner.is_none(py);
+        // SAFETY: we can ensure that validity has correct capacity.
+        unsafe { validity.push_unchecked(is_valid) };
+    }
+    ObjectChunked::<ObjectValue>::new_from_vec_and_validity(
+        name,
+        objects,
+        validity.into_opt_validity(),
+    )
+    .into_series()
 }
 
 #[pymethods]
@@ -75,18 +97,14 @@ impl PySeries {
         array: &Bound<PyArray1<pf16>>,
         nan_is_null: bool,
     ) -> PyResult<Self> {
+        let arr = numpy_array_to_arrow(array);
         if nan_is_null {
-            let array = array.readonly();
-            let vals = array.as_slice().unwrap();
             py.enter_polars_series(|| {
-                let ca: Float16Chunked = vals
-                    .iter()
-                    .map(|&val| if pf16::is_nan(val) { None } else { Some(val) })
-                    .collect_trusted();
-                Ok(ca.with_name(name.into()))
+                let validity = polars_compute::nan::is_not_nan(arr.values());
+                Ok(Series::from_array(name.into(), arr.with_validity(validity)))
             })
         } else {
-            Ok(mmap_numpy_array(name, array))
+            Ok(Series::from_array(name.into(), arr).into())
         }
     }
 
@@ -97,18 +115,14 @@ impl PySeries {
         array: &Bound<PyArray1<f32>>,
         nan_is_null: bool,
     ) -> PyResult<Self> {
+        let arr = numpy_array_to_arrow(array);
         if nan_is_null {
-            let array = array.readonly();
-            let vals = array.as_slice().unwrap();
             py.enter_polars_series(|| {
-                let ca: Float32Chunked = vals
-                    .iter()
-                    .map(|&val| if f32::is_nan(val) { None } else { Some(val) })
-                    .collect_trusted();
-                Ok(ca.with_name(name.into()))
+                let validity = polars_compute::nan::is_not_nan(arr.values());
+                Ok(Series::from_array(name.into(), arr.with_validity(validity)))
             })
         } else {
-            Ok(mmap_numpy_array(name, array))
+            Ok(Series::from_array(name.into(), arr).into())
         }
     }
 
@@ -119,18 +133,14 @@ impl PySeries {
         array: &Bound<PyArray1<f64>>,
         nan_is_null: bool,
     ) -> PyResult<Self> {
+        let arr = numpy_array_to_arrow(array);
         if nan_is_null {
-            let array = array.readonly();
-            let vals = array.as_slice().unwrap();
             py.enter_polars_series(|| {
-                let ca: Float64Chunked = vals
-                    .iter()
-                    .map(|&val| if f64::is_nan(val) { None } else { Some(val) })
-                    .collect_trusted();
-                Ok(ca.with_name(name.into()))
+                let validity = polars_compute::nan::is_not_nan(arr.values());
+                Ok(Series::from_array(name.into(), arr.with_validity(validity)))
             })
         } else {
-            Ok(mmap_numpy_array(name, array))
+            Ok(Series::from_array(name.into(), arr).into())
         }
     }
 }
@@ -358,20 +368,7 @@ impl PySeries {
     pub fn new_object(py: Python<'_>, name: &str, values: Vec<ObjectValue>, _strict: bool) -> Self {
         #[cfg(feature = "object")]
         {
-            let mut validity = BitmapBuilder::with_capacity(values.len());
-            values.iter().for_each(|v| {
-                let is_valid = !v.inner.is_none(py);
-                // SAFETY: we can ensure that validity has correct capacity.
-                unsafe { validity.push_unchecked(is_valid) };
-            });
-            // Object builder must be registered. This is done on import.
-            let ca = ObjectChunked::<ObjectValue>::new_from_vec_and_validity(
-                name.into(),
-                values,
-                validity.into_opt_validity(),
-            );
-            let s = ca.into_series();
-            s.into()
+            PySeries::from(series_from_objects(py, name.into(), values))
         }
         #[cfg(not(feature = "object"))]
         panic!("activate 'object' feature")
