@@ -11,8 +11,8 @@ use polars_ops::frame::JoinArgs;
 use polars_plan::dsl::deletion::DeletionFilesList;
 use polars_plan::dsl::{
     CastColumnsPolicy, FileSinkOptions, JoinTypeOptionsIR, MissingColumnsPolicy,
-    PartitionTargetCallback, PartitionVariantIR, PartitionedSinkOptionsIR, ScanSources,
-    SinkFinishCallback, SinkOptions, SinkTarget, SortColumnIR, TableStatistics,
+    PartitionTargetCallback, PartitionVariantIR, PartitionedSinkOptionsIR, PredicateFileSkip,
+    ScanSources, SinkFinishCallback, SinkOptions, SortColumnIR, TableStatistics,
 };
 use polars_plan::plans::hive::HivePartitionsDf;
 use polars_plan::plans::{AExpr, DataFrameUdf, IR};
@@ -28,7 +28,7 @@ mod to_graph;
 pub mod visualization;
 
 pub use fmt::visualize_plan;
-use polars_plan::prelude::{FileType, PlanCallback};
+use polars_plan::prelude::{FileWriteFormat, PlanCallback};
 #[cfg(feature = "dynamic_group_by")]
 use polars_time::DynamicGroupOptions;
 use polars_time::{ClosedWindow, Duration};
@@ -50,6 +50,12 @@ slotmap::new_key_type! {
     pub struct PhysNodeKey;
 }
 
+impl PhysNodeKey {
+    pub fn as_ffi(&self) -> u64 {
+        self.0.as_ffi()
+    }
+}
+
 /// A node in the physical plan.
 ///
 /// A physical plan is created when the `IR` is translated to a directed
@@ -191,11 +197,8 @@ pub enum PhysNodeKind {
     },
 
     FileSink {
-        target: SinkTarget,
-        sink_options: SinkOptions,
-        file_type: FileType,
         input: PhysStream,
-        cloud_options: Option<CloudOptions>,
+        options: FileSinkOptions,
     },
 
     PartitionedSink {
@@ -204,17 +207,12 @@ pub enum PhysNodeKind {
         file_path_cb: Option<PartitionTargetCallback>,
         sink_options: SinkOptions,
         variant: PartitionVariantIR,
-        file_type: FileType,
+        file_type: FileWriteFormat,
         cloud_options: Option<CloudOptions>,
         per_partition_sort_by: Option<Vec<SortColumnIR>>,
         finish_callback: Option<SinkFinishCallback>,
     },
 
-    FileSink2 {
-        input: PhysStream,
-        options: FileSinkOptions,
-    },
-
     PartitionedSink2 {
         input: PhysStream,
         options: PartitionedSinkOptionsIR,
@@ -317,7 +315,7 @@ pub enum PhysNodeKind {
         row_index: Option<RowIndex>,
         pre_slice: Option<Slice>,
         predicate: Option<ExprIR>,
-        predicate_file_skip_applied: Option<bool>,
+        predicate_file_skip_applied: Option<PredicateFileSkip>,
 
         hive_parts: Option<HivePartitionsDf>,
         include_file_paths: Option<PlSmallStr>,
@@ -338,10 +336,11 @@ pub enum PhysNodeKind {
     },
 
     GroupBy {
-        input: PhysStream,
-        key: Vec<ExprIR>,
+        inputs: Vec<PhysStream>,
+        // Must have the same schema when applied for each input.
+        key_per_input: Vec<Vec<ExprIR>>,
         // Must be a 'simple' expression, a singular column feeding into a single aggregate, or Len.
-        aggs: Vec<ExprIR>,
+        aggs_per_input: Vec<Vec<ExprIR>>,
     },
 
     #[cfg(feature = "dynamic_group_by")]
@@ -456,7 +455,6 @@ fn visit_node_inputs_mut(
             | PhysNodeKind::InMemorySink { input }
             | PhysNodeKind::CallbackSink { input, .. }
             | PhysNodeKind::FileSink { input, .. }
-            | PhysNodeKind::FileSink2 { input, .. }
             | PhysNodeKind::PartitionedSink { input, .. }
             | PhysNodeKind::PartitionedSink2 { input, .. }
             | PhysNodeKind::InMemoryMap { input, .. }
@@ -467,8 +465,7 @@ fn visit_node_inputs_mut(
             | PhysNodeKind::GatherEvery { input, .. }
             | PhysNodeKind::Rle(input)
             | PhysNodeKind::RleId(input)
-            | PhysNodeKind::PeakMinMax { input, .. }
-            | PhysNodeKind::GroupBy { input, .. } => {
+            | PhysNodeKind::PeakMinMax { input, .. } => {
                 rec!(input.node);
                 visit(input);
             },
@@ -572,7 +569,9 @@ fn visit_node_inputs_mut(
                 visit(repeats);
             },
 
-            PhysNodeKind::OrderedUnion { inputs } | PhysNodeKind::Zip { inputs, .. } => {
+            PhysNodeKind::GroupBy { inputs, .. }
+            | PhysNodeKind::OrderedUnion { inputs }
+            | PhysNodeKind::Zip { inputs, .. } => {
                 for input in inputs {
                     rec!(input.node);
                     visit(input);
