@@ -37,6 +37,7 @@ pub mod plpath;
 pub mod priority;
 pub mod regex_cache;
 pub mod relaxed_cell;
+pub mod reuse_vec;
 pub mod row_counter;
 pub mod select;
 pub mod slice;
@@ -87,3 +88,4 @@ pub mod kahan_sum;
 pub use either;
 pub use idx_vec::UnitVec;
 pub mod chunked_bytes_cursor;
+pub mod concat_vec;
