@@ -193,7 +193,12 @@ def test_write_delta(df: pl.DataFrame, tmp_path: Path) -> None:
 
     assert tbl.version() == 1
     assert partitioned_tbl.version() == 0
-    assert Path(partitioned_tbl.table_uri) == partitioned_tbl_uri
+
+    uri = partitioned_tbl.table_uri.removeprefix("file://")
+    if os.name == "nt" and uri.startswith("/"):
+        uri = uri[1:]
+
+    assert Path(uri) == partitioned_tbl_uri
     assert partitioned_tbl.metadata().partition_columns == ["strings"]
 
     assert_frame_equal(v0, pl_df_0, check_row_order=False)
@@ -226,6 +231,93 @@ def test_write_delta(df: pl.DataFrame, tmp_path: Path) -> None:
     df_supported.write_delta(partitioned_tbl_uri, mode="overwrite")
 
 
+@pytest.mark.write_disk
+def test_sink_delta(df: pl.DataFrame, tmp_path: Path) -> None:
+    v0 = df.lazy().select(pl.col(pl.String))
+    v1 = df.lazy().select(pl.col(pl.Int64))
+    df_supported = df.drop(["cat", "enum", "time"])
+
+    # Case: Success (version 0)
+    v0.sink_delta(tmp_path)
+
+    # Case: Error if table exists
+    with pytest.raises(DeltaError, match="A table already exists"):
+        v0.sink_delta(tmp_path)
+
+    # Case: Overwrite with new version (version 1)
+    v1.sink_delta(
+        tmp_path, mode="overwrite", delta_write_options={"schema_mode": "overwrite"}
+    )
+
+    # Case: Error if schema contains unsupported columns
+    with pytest.raises(TypeError):
+        df.lazy().sink_delta(
+            tmp_path, mode="overwrite", delta_write_options={"schema_mode": "overwrite"}
+        )
+
+    partitioned_tbl_uri = (tmp_path / ".." / "partitioned_table_sink").resolve()
+
+    # Case: Write new partitioned table (version 0)
+    df_supported.lazy().sink_delta(
+        partitioned_tbl_uri, delta_write_options={"partition_by": "strings"}
+    )
+
+    # Case: Read back
+    tbl = DeltaTable(tmp_path)
+    partitioned_tbl = DeltaTable(partitioned_tbl_uri)
+
+    pl_df_0 = pl.read_delta(tbl.table_uri, version=0)
+    pl_df_1 = pl.read_delta(tbl.table_uri, version=1)
+    pl_df_partitioned = pl.read_delta(partitioned_tbl_uri)
+
+    assert v0.collect().shape == pl_df_0.shape
+    assert v0.collect_schema().names() == pl_df_0.columns
+    assert v1.collect().shape == pl_df_1.shape
+    assert v1.collect_schema().names() == pl_df_1.columns
+
+    assert df_supported.shape == pl_df_partitioned.shape
+    assert sorted(df_supported.columns) == sorted(pl_df_partitioned.columns)
+
+    assert tbl.version() == 1
+    assert partitioned_tbl.version() == 0
+
+    uri = partitioned_tbl.table_uri.removeprefix("file://")
+    if os.name == "nt" and uri.startswith("/"):
+        uri = uri[1:]
+
+    assert Path(uri) == partitioned_tbl_uri
+    assert partitioned_tbl.metadata().partition_columns == ["strings"]
+
+    assert_frame_equal(v0.collect(), pl_df_0, check_row_order=False)
+    assert_frame_equal(v1.collect(), pl_df_1, check_row_order=False)
+
+    cols = [c for c in df_supported.columns if not c.startswith("list_")]
+    assert_frame_equal(
+        df_supported.select(cols),
+        pl_df_partitioned.select(cols),
+        check_row_order=False,
+    )
+
+    # Case: Append to existing tables
+    v1.sink_delta(tmp_path, mode="append")
+    tbl = DeltaTable(tmp_path)
+    pl_df_1 = pl.read_delta(tbl.table_uri, version=2)
+
+    assert tbl.version() == 2
+    assert pl_df_1.shape == (6, 2)  # Rows are doubled
+    assert v1.collect_schema().names() == pl_df_1.columns
+
+    df_supported.lazy().sink_delta(partitioned_tbl_uri, mode="append")
+    partitioned_tbl = DeltaTable(partitioned_tbl_uri)
+    pl_df_partitioned = pl.read_delta(partitioned_tbl.table_uri, version=1)
+
+    assert partitioned_tbl.version() == 1
+    assert pl_df_partitioned.shape == (6, 14)  # Rows are doubled
+    assert sorted(df_supported.columns) == sorted(pl_df_partitioned.columns)
+
+    df_supported.lazy().sink_delta(partitioned_tbl_uri, mode="overwrite")
+
+
 @pytest.mark.write_disk
 def test_write_delta_overwrite_schema_deprecated(
     df: pl.DataFrame, tmp_path: Path
@@ -732,7 +824,7 @@ def test_scan_delta_loads_aws_profile_endpoint_url(
 [profile endpoint_333]
 aws_access_key_id=A
 aws_secret_access_key=A
-endpoint_url = http://localhost:333
+endpoint_url = http://127.0.0.1:54321
 """)
 
     monkeypatch.setenv("AWS_CONFIG_FILE", str(cfg_file_path))
@@ -752,11 +844,11 @@ endpoint_url = http://localhost:333
     assert provider._can_use_as_provider()
 
     assert provider._storage_update_options() == {
-        "endpoint_url": "http://localhost:333"
+        "endpoint_url": "http://127.0.0.1:54321"
     }
 
-    with pytest.raises(DeltaError, match="http://localhost:333"):
+    with pytest.raises((DeltaError, OSError), match=r"http://127.0.0.1:54321"):
         pl.scan_delta("s3://.../...")
 
-    with pytest.raises(DeltaError, match="http://localhost:333"):
+    with pytest.raises((DeltaError, OSError), match=r"http://127.0.0.1:54321"):
         pl.DataFrame({"x": 1}).write_delta("s3://.../...", mode="append")
