@@ -1,12 +1,15 @@
 from __future__ import annotations
 
-from typing import Callable
+from typing import TYPE_CHECKING
 
 import pytest
 
 import polars as pl
 from polars.testing import assert_series_equal
 
+if TYPE_CHECKING:
+    from collections.abc import Callable
+
 
 def set_nulls(s: pl.Series, mask: list[bool]) -> pl.Series:
     return pl.select(pl.when(pl.Series(mask)).then(s).alias(s.name)).to_series()
